{****************************************************************}
{                                                                }
{ PSC HideComp Expert                                            }
{ Copyright (c) 1999 puterSoft.Com, Inc.                         }
{                                                                }
{ e-mail: contact@puterSoft.Com                                  }
{    WWW: http://www.puterSoft.Com                               }
{                                                                }
{****************************************************************}

unit PSCHideExp;

interface

uses
  Windows, SysUtils, Classes, Forms, Menus, Controls,
  ToolIntf, ExptIntf, DsgnIntf, TypInfo;

type

{----------------------------------------------------------}
{ TPSCExpert }

  TPSCExpert = class(TIExpert)
  public
    function GetName: string; override;
    function GetStyle: TExpertStyle; override;
    function GetIDString: string; override;
    function GetComment: string; override;
    function GetAuthor: string; override;
    function GetPage: string; override;
    function GetState: TExpertState; override;
    function GetMenuText: string; override;
    function GetGlyph: HICON; override;
    procedure Execute; override;
  end;

{----------------------------------------------------------}
{ TPSCHideCompExpert }

  TPSCSubMenuExpert = class(TPSCExpert)
  private
    SubMenuItem: TIMenuItemIntf;
  public
    constructor Create;
    destructor Destroy; override;
  end;

{----------------------------------------------------------}
{ TPSCHideCompExpert }

  TPSCHideCompExpert = class(TPSCExpert)
  private
    HideItem: TIMenuItemIntf;
    ShowItem: TIMenuItemIntf;
    procedure HideItemClick(Sender: TIMenuItemIntf);
    procedure ShowItemClick(Sender: TIMenuItemIntf);
  public
    constructor Create;
    destructor Destroy; override;
  end;

{----------------------------------------------------------}
implementation
{----------------------------------------------------------}

const
  SDelphiMenu = 'ToolsMenu';//don't resource
  SPSCSubMenu = 'PSCSubMenu';//don't resource
  SPSCSubMenuCaption ='&PSC Experts';

var
  _PSCHideCompExpert: TPSCHideCompExpert;
  _PSCSubMenuExpert : TPSCSubMenuExpert;

{----------------------------------------------------------}

function ExpertInsertMenuItemEx(const MenuItemName: string; AIndex: Integer;
  ACaption, AName, AHint: string; AShortCut: Integer; AFlags: TIMenuFlags;
  AEventHandler: TIMenuClickEvent): TIMenuItemIntf;
var
  MainMenu: TIMainMenuIntf;
  InToMenu: TIMenuItemIntf;
  MenuItems: TIMenuItemIntf;
begin
  Result := nil;
  if ToolServices = nil then
    Exit;
  MainMenu := ToolServices.GetMainMenu;
  if MainMenu <> nil then
  try
    MenuItems := MainMenu.GetMenuItems;
    if MenuItems <> nil then
    try
      InToMenu := MainMenu.FindMenuItem(MenuItemName); //don't resource
      if InToMenu <> nil then
      try
        with InToMenu do
        begin
          if AIndex < 0
           then AIndex := GetItemCount - 1;
          Result := InsertItem(AIndex, ACaption,
            AName, AHint, AShortCut, 0, 0,
            [mfVisible, mfEnabled], AEventHandler);
        end;
      finally
        InToMenu.Free;
      end;
    finally
      MenuItems.Free;
    end;
  finally
    MainMenu.Free;
  end;
end;

{--------------------------------------}

function ExpertInsertMenuItemAboveSeparator(const AMenuItemName: string;
  ACaption, AName, AHint: string; AShortCut: Integer; AFlags: TIMenuFlags;
  AEventHandler: TIMenuClickEvent): TIMenuItemIntf;
var
  MainMenu: TIMainMenuIntf;
  InToMenu: TIMenuItemIntf;
  MenuItems: TIMenuItemIntf;
  InsertIndex: Integer;

  procedure _FindSeparator;
  var
    i: Integer;
  begin
    InsertIndex := -1;
    with InToMenu do
      for i := GetItemCount-1 downto 0 do
        with GetItem(i) do
          if GetCaption = '-' then
          begin
            InsertIndex := i - 1;
            Free;
            Break;
          end
          else Free;
  end;

begin
  Result := nil;
  if ToolServices = nil then
    Exit;
  MainMenu := ToolServices.GetMainMenu;
  if MainMenu <> nil then
  try
    MenuItems := MainMenu.GetMenuItems;
    if MenuItems <> nil then
    try
      InToMenu := MainMenu.FindMenuItem(AMenuItemName); //don't resource
      if InToMenu <> nil then
      try
        with InToMenu do
        begin
          _FindSeparator;

          if InsertIndex < 0
           then InsertIndex := GetItemCount - 1;

          Result := InsertItem(InsertIndex, ACaption,
            AName, AHint, AShortCut, 0, 0,
            [mfVisible, mfEnabled], AEventHandler);
        end;
      finally
        InToMenu.Free;
      end;
    finally
      MenuItems.Free;
    end;
  finally
    MainMenu.Free;
  end;
end;

{--------------------------------------}

function ExpertInsertMenuSubMenu(const AMenuItemName, ACaption, AName: string;
  AIndex: Integer): TIMenuItemIntf;
begin
  Result := ExpertInsertMenuItemEx(AMenuItemName, AIndex, ACaption, AName, '',
    0, [mfBarBreak], nil);
end;

{--------------------------------------}

function ExpertInsertMenuSeparator(const AMenuItemName: string;
  AIndex: Integer): TIMenuItemIntf;
begin
  Result := ExpertInsertMenuItemEx(AMenuItemName, AIndex, '-', '', '',
    0, [mfBreak], nil);
end;

{--------------------------------------}
function ExpertInsertMenuItem(const AMenuItemName: string; AIndex: Integer;
  ACaption, AName, AHint: string; AShortCut: Integer;
  AEventHandler: TIMenuClickEvent): TIMenuItemIntf;
begin
  Result := ExpertInsertMenuItemEx(AMenuItemName, AIndex, ACaption, AName,
    AHint, AShortCut, [mfVisible, mfEnabled], AEventHandler);
end;

{--------------------------------------}

function GetDesignedForm: TCustomForm;
begin
  Result := nil;
  if Screen.CustomFormCount < 2 then
    Exit;

  Result := Screen.CustomForms[0];
  if not (csDesigning in Result.ComponentState) then
  begin
    Result := Screen.CustomForms[1];
    if not (csDesigning in Result.ComponentState) then
      Exit;
  end;
end;

{--------------------------------------}

procedure SetContainresVisible(Form: TCustomForm; AVisible: Boolean);
var
  WinHandle: THandle;
  AStr: array[0..255] of Char;
  R: TRect;
begin
  WinHandle := GetWindow(Form.Handle, GW_CHILD);
  WinHandle := GetWindow(WinHandle, GW_HWNDLAST);

  while (WinHandle <> 0) do
  begin
    GetClassName(WinHandle, AStr, 254);
    if (StrComp(AStr, 'TContainer') = 0) then //don't resource
    begin
      Windows.GetClientRect(WinHandle, r);

      with R do
      begin
        if AVisible then
          ShowWindow(WinHandle, SW_SHOW)
        else
          ShowWindow(WinHandle, SW_HIDE);
      end;
    end;
    WinHandle := GetWindow(WinHandle, GW_HWNDPREV);
  end;
end;

{--------------------------------------}

procedure SetSelCompsVisible(AVisible: Boolean);
var
  Form: TCustomForm;
begin
  Form := GetDesignedForm;
  if Form = nil then
    Exit;
  SetContainResVisible(Form, AVisible);
end;


{----------------------------------------------------------}
{ TPSCExpert }

function TPSCExpert.GetName: string;
begin
  Result := ClassName + ' Expert'; //don't resource
end;

{--------------------------------------}

function TPSCExpert.GetStyle: TExpertStyle;
begin
  Result := esAddIn;
end;

{--------------------------------------}

function TPSCExpert.GetIDString: string;
begin
  Result := 'puterSoft.' + ClassName; //don't resource
end;

{--------------------------------------}

function TPSCExpert.GetComment: string;
begin
  Result := '';
end;

{--------------------------------------}

function TPSCExpert.GetAuthor: string;
begin
  Result := 'puterSoft.Com, Inc.';
end;

{--------------------------------------}

function TPSCExpert.GetPage: string;
begin
  Result := '';
end;

{--------------------------------------}

function TPSCExpert.GetState: TExpertState;
begin
  Result := [esEnabled];
end;

{--------------------------------------}

function TPSCExpert.GetMenuText: string;
begin
  Result := '';
end;

{--------------------------------------}

function TPSCExpert.GetGlyph: HICON;
begin
  Result := 0;
end;

{--------------------------------------}

procedure TPSCExpert.Execute;
begin
end;


{-----------------------------------------------------------}
{ TPSCSubMenuExpert }

constructor TPSCSubMenuExpert.Create;
begin
  inherited Create;
  SubMenuItem := ExpertInsertMenuItemAboveSeparator(SDelphiMenu,
    SPSCSubMenuCaption, SPSCSubMenu, '', 0, [mfBarBreak], nil);
end;

destructor TPSCSubMenuExpert.Destroy;
begin
  SubMenuItem.Free;
  inherited;
end;


{----------------------------------------------------------}
{ TPSCHideCompExpert }

const
  SHideNonVisual = '&Hide Non-visual';//don't resource
  SShowNonVisual = '&Show Non-visual';//don't resource
  SHideNonVisualName = 'PSCHideCompExpertHide';//don't resource
  SShowNonVisualName = 'PSCHideCompExpertShow';//don't resource
  SHideNonVisualHint = 'Hides all non-visual components';//don't resource
  SShowNonVisualHint = 'Shows all non-visual components';//don't resource
  SHideNonVisualKey = 'Shift+Ctrl+Alt+H';//don't resource
  SShowNonVisualKey = 'Shift+Ctrl+Alt+S';//don't resource

constructor TPSCHideCompExpert.Create;
begin
  inherited Create;
  ShowItem := ExpertInsertMenuItem(SPSCSubMenu, 0, SShowNonVisual,
    SShowNonVisualName, SShowNonVisualHint, TextToShortCut(SShowNonVisualKey),
    ShowItemClick);
  HideItem := ExpertInsertMenuItem(SPSCSubMenu, 0, SHideNonVisual,
    SHideNonVisualName, SHideNonVisualHint, TextToShortCut(SHideNonVisualKey),
    HideItemClick);
end;

{--------------------------------------}

destructor TPSCHideCompExpert.Destroy;
begin
  ShowItem.Free;
  HideItem.Free;
  inherited Destroy;
end;

{--------------------------------------}

procedure TPSCHideCompExpert.HideItemClick(Sender: TIMenuItemIntf);
begin
  SetSelCompsVisible(False);
end;

{--------------------------------------}

procedure TPSCHideCompExpert.ShowItemClick(Sender: TIMenuItemIntf);
begin
  SetSelCompsVisible(True);
end;


{----------------------------------------------------------}

procedure InitializeSubMenuInspector;
begin
  _PSCSubMenuExpert := TPSCSubMenuExpert.Create;
  RegisterLibraryExpert(_PSCSubMenuExpert);
end;

{--------------------------------------}

procedure FinalizeSubMenuInspector;
begin
end;

{----------------------------------------------------------}
{$IFDEF VER130}
var
  WHook: THandle = 0;

function CallWndProcHook(Code, WParam, LParam: Integer): HResult; stdcall;
begin
  Result := CallNextHookEx(Whook, Code, WParam, LParam);

  if (KeyDataToShiftState(LParam) = [ssShift, ssCtrl, ssAlt]) and
    Assigned(_PSCHideCompExpert) then
  begin
    case Word(wParam) of
      word('H'): _PSCHideCompExpert.HideItemClick(nil);
      word('S'): _PSCHideCompExpert.ShowItemClick(nil);
    end;
  end;
end;
{$ENDIF VER130}

{--------------------------------------}

procedure InitializeHideCompInspector;
begin
  _PSCHideCompExpert := TPSCHideCompExpert.Create;
  RegisterLibraryExpert(_PSCHideCompExpert);

{$IFDEF VER130}
  WHook := SetWindowsHookEx(WH_KEYBOARD, @CallWndProcHook, 0, GetCurrentThreadId);
{$ENDIF VER130}
end;

{--------------------------------------}

procedure FinalizeHideCompInspector;
begin
{$IFDEF VER130}
  if WHook <> 0
   then UnhookWindowsHookEx(WHook);
{$ENDIF VER130}
end;


{----------------------------------------------------------}
initialization
  InitializeSubMenuInspector;
  InitializeHideCompInspector;

{----------------------------------------------------------}
finalization
  FinalizeHideCompInspector;
  FinalizeSubMenuInspector;

end.

